@@ -31,6 +31,7 @@ from .. import exc
 __all__ = ['VisitableType', 'Visitable', 'ClauseVisitor',
     'CloningVisitor', 'ReplacingCloningVisitor', 'iterate',
     'iterate_depthfirst', 'traverse_using', 'traverse',
+    'traverse_depthfirst',
     'cloned_traverse', 'replacement_traverse']
 
 
@@ -50,11 +51,9 @@ class VisitableType(type):
     Classes having no __visit_name__ attribute will remain unaffected.
     """
     def __init__(cls, clsname, bases, clsdict):
-        if cls.__name__ == 'Visitable' or not hasattr(cls, '__visit_name__'):
-            super(VisitableType, cls).__init__(clsname, bases, clsdict)
-            return
-
-        _generate_dispatch(cls)
+        if clsname != 'Visitable' and \
+                hasattr(cls, '__visit_name__'):
+            _generate_dispatch(cls)
 
         super(VisitableType, cls).__init__(clsname, bases, clsdict)
 
@@ -98,14 +97,12 @@ def _generate_dispatch(cls):
         cls._compiler_dispatch = _compiler_dispatch
 
 
-class Visitable(object):
+class Visitable(util.with_metaclass(VisitableType, object)):
     """Base class for visitable objects, applies the
     ``VisitableType`` metaclass.
 
     """
 
-    __metaclass__ = VisitableType
-
 
 class ClauseVisitor(object):
     """Base class for visitor objects which can traverse using
@@ -270,8 +267,8 @@ def cloned_traverse(obj, opts, visitors):
     """clone the given expression structure, allowing
     modifications by visitors."""
 
-    cloned = util.column_dict()
-    stop_on = util.column_set(opts.get('stop_on', []))
+    cloned = {}
+    stop_on = set(opts.get('stop_on', []))
 
     def clone(elem):
         if elem in stop_on:
@@ -294,8 +291,8 @@ def replacement_traverse(obj, opts, replace):
     """clone the given expression structure, allowing element
     replacement by a given replacement function."""
 
-    cloned = util.column_dict()
-    stop_on = util.column_set([id(x) for x in opts.get('stop_on', [])])
+    cloned = {}
+    stop_on = set([id(x) for x in opts.get('stop_on', [])])
 
     def clone(elem, **kw):
         if id(elem) in stop_on or \
