@@ -10,8 +10,12 @@ from .warnings import testing_warn, assert_warnings, resetwarnings
 from . import config
 
 from .exclusions import db_spec, _is_excluded, fails_if, skip_if, future,\
-    fails_on, fails_on_everything_except, skip, only_on, exclude, against,\
-    _server_version, only_if
+    fails_on, fails_on_everything_except, skip, only_on, exclude, \
+    against as _against, _server_version, only_if
+
+
+def against(*queries):
+    return _against(config._current, *queries)
 
 from .assertions import emits_warning, emits_warning_on, uses_deprecated, \
         eq_, ne_, is_, is_not_, startswith_, assert_raises, \
@@ -22,6 +26,7 @@ from .util import run_as_contextmanager, rowset, fail, provide_metadata, adict
 
 crashes = skip
 
-from .config import db, requirements as requires
+from .config import db
+from .config import requirements as requires
 
 from . import mock
\ No newline at end of file
