@@ -1,4 +1,3 @@
-from __future__ import with_statement
 from .. import fixtures, config
 from ..config import requirements
 from .. import exclusions
@@ -13,6 +12,8 @@ from ..schema import Table, Column
 class LastrowidTest(fixtures.TablesTest):
     run_deletes = 'each'
 
+    __backend__ = True
+
     __requires__ = 'implements_get_lastrowid', 'autoincrement_insert'
 
     __engine_options__ = {"implicit_returning": False}
@@ -57,8 +58,9 @@ class LastrowidTest(fixtures.TablesTest):
             [pk]
         )
 
-    @exclusions.fails_if(lambda: util.pypy, "lastrowid not maintained after "
-                            "connection close")
+    # failed on pypy1.9 but seems to be OK on pypy 2.1
+    #@exclusions.fails_if(lambda: util.pypy, "lastrowid not maintained after "
+    #                        "connection close")
     @requirements.dbapi_lastrowid
     def test_native_lastrowid_autoinc(self):
         r = config.db.execute(
@@ -74,6 +76,7 @@ class LastrowidTest(fixtures.TablesTest):
 
 class InsertBehaviorTest(fixtures.TablesTest):
     run_deletes = 'each'
+    __backend__ = True
 
     @classmethod
     def define_tables(cls, metadata):
@@ -156,8 +159,9 @@ class InsertBehaviorTest(fixtures.TablesTest):
         )
 
 class ReturningTest(fixtures.TablesTest):
-    run_deletes = 'each'
+    run_create_tables = 'each'
     __requires__ = 'returning', 'autoincrement_insert'
+    __backend__ = True
 
     __engine_options__ = {"implicit_returning": True}
 
