@@ -5,10 +5,12 @@
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
 from .compat import callable, cmp, reduce,  \
-    threading, py3k, py33, py2k, py3k_warning, jython, pypy, cpython, win32, \
-    set_types, py26, \
-    pickle, dottedgetter, parse_qsl, namedtuple, next, WeakSet, reraise, \
-    raise_from_cause, u, b, ue, string_types, text_type, int_types, binary_type
+    threading, py3k, py33, py2k, jython, pypy, cpython, win32, \
+    pickle, dottedgetter, parse_qsl, namedtuple, next, reraise, \
+    raise_from_cause, text_type, string_types, int_types, binary_type, \
+    quote_plus, with_metaclass, print_, itertools_filterfalse, u, ue, b,\
+    unquote_plus, unquote, b64decode, b64encode, byte_buffer, itertools_filter,\
+    iterbytes, StringIO, inspect_getargspec
 
 from ._collections import KeyedTuple, ImmutableContainer, immutabledict, \
     Properties, OrderedProperties, ImmutableProperties, OrderedDict, \
@@ -16,23 +18,24 @@ from ._collections import KeyedTuple, ImmutableContainer, immutabledict, \
     column_dict, ordered_column_set, populate_column_dict, unique_list, \
     UniqueAppender, PopulateDict, EMPTY_SET, to_list, to_set, \
     to_column_set, update_copy, flatten_iterator, \
-    LRUCache, ScopedRegistry, ThreadLocalRegistry, WeakSequence
+    LRUCache, ScopedRegistry, ThreadLocalRegistry, WeakSequence, \
+    coerce_generator_arg
 
 from .langhelpers import iterate_attributes, class_hierarchy, \
     portable_instancemethod, unbound_method_to_callable, \
     getargspec_init, format_argspec_init, format_argspec_plus, \
     get_func_kwargs, get_cls_kwargs, decorator, as_interface, \
     memoized_property, memoized_instancemethod, md5_hex, \
-    group_expirable_memoized_property, importlater, decode_slice, \
+    group_expirable_memoized_property, dependencies, decode_slice, \
     monkeypatch_proxied_specials, asbool, bool_or_str, coerce_kw_type,\
     duck_type_collection, assert_arg_type, symbol, dictlike_iteritems,\
     classproperty, set_creation_order, warn_exception, warn, NoneType,\
     constructor_copy, methods_equivalent, chop_traceback, asint,\
     generic_repr, counter, PluginLoader, hybridmethod, safe_reraise,\
-    only_once
+    get_callable_argspec, only_once
 
 from .deprecations import warn_deprecated, warn_pending_deprecation, \
-    deprecated, pending_deprecation
+    deprecated, pending_deprecation, inject_docstring_text
 
 # things that used to be not always available,
 # but are now as of current support Python versions
