@@ -218,7 +218,7 @@ class AttributesTest(fixtures.ORMTest):
         attributes.register_attribute(MyTest2, 'b', uselist=False,
                 useobject=False)
 
-        # shouldn't be pickling callables at the class level
+        # shouldnt be pickling callables at the class level
 
         def somecallable(state, passive):
             return None
@@ -236,21 +236,6 @@ class AttributesTest(fixtures.ORMTest):
         o2 = pickle.loads(pk_o)
         pk_o2 = pickle.dumps(o2)
 
-        # so... pickle is creating a new 'mt2' string after a roundtrip here,
-        # so we'll brute-force set it to be id-equal to the original string
-        if False:
-            o_mt2_str = [ k for k in o.__dict__ if k == 'mt2'][0]
-            o2_mt2_str = [ k for k in o2.__dict__ if k == 'mt2'][0]
-            self.assert_(o_mt2_str == o2_mt2_str)
-            self.assert_(o_mt2_str is not o2_mt2_str)
-            # change the id of o2.__dict__['mt2']
-            former = o2.__dict__['mt2']
-            del o2.__dict__['mt2']
-            o2.__dict__[o_mt2_str] = former
-
-            # Relies on dict ordering
-            if not jython:
-                self.assert_(pk_o == pk_o2)
 
         # the above is kind of distrurbing, so let's do it again a little
         # differently.  the string-id in serialization thing is just an
@@ -261,11 +246,6 @@ class AttributesTest(fixtures.ORMTest):
         o3 = pickle.loads(pk_o2)
         pk_o3 = pickle.dumps(o3)
         o4 = pickle.loads(pk_o3)
-        pk_o4 = pickle.dumps(o4)
-
-        # Relies on dict ordering
-        if not jython:
-            self.assert_(pk_o3 == pk_o4)
 
         # and lastly make sure we still have our data after all that.
         # identical serialzation is great, *if* it's complete :)
@@ -294,6 +274,7 @@ class AttributesTest(fixtures.ORMTest):
         assert state.obj() is None
         assert state.dict == {}
 
+    @testing.requires.predictable_gc
     def test_object_dereferenced_error(self):
         class Foo(object):
             pass
@@ -317,7 +298,8 @@ class AttributesTest(fixtures.ORMTest):
         )
 
     def test_deferred(self):
-        class Foo(object):pass
+        class Foo(object):
+            pass
 
         data = {'a':'this is a', 'b':12}
         def loader(state, keys):
@@ -848,7 +830,7 @@ class AttributesTest(fixtures.ORMTest):
             attributes.register_attribute(Foo, 'collection',
                     uselist=True, typecallable=dict, useobject=True)
             assert False
-        except sa_exc.ArgumentError, e:
+        except sa_exc.ArgumentError as e:
             assert str(e) \
                 == 'Type InstrumentedDict must elect an appender '\
                 'method to be a collection class'
@@ -875,7 +857,7 @@ class AttributesTest(fixtures.ORMTest):
             attributes.register_attribute(Foo, 'collection',
                     uselist=True, typecallable=MyColl, useobject=True)
             assert False
-        except sa_exc.ArgumentError, e:
+        except sa_exc.ArgumentError as e:
             assert str(e) \
                 == 'Type MyColl must elect an appender method to be a '\
                 'collection class'
@@ -899,7 +881,7 @@ class AttributesTest(fixtures.ORMTest):
         try:
             Foo().collection
             assert True
-        except sa_exc.ArgumentError, e:
+        except sa_exc.ArgumentError as e:
             assert False
 
 class GetNoValueTest(fixtures.ORMTest):
@@ -1162,12 +1144,8 @@ class BackrefTest(fixtures.ORMTest):
         p2.children.append(c1)
         assert c1.parent is p2
 
-        # note its still in p1.children -
-        # the event model currently allows only
-        # one level deep.  without the parent_token,
-        # it keeps going until a ValueError is raised
-        # and this condition changes.
-        assert c1 in p1.children
+        # event propagates to remove as of [ticket:2789]
+        assert c1 not in p1.children
 
 class CyclicBackrefAssertionTest(fixtures.TestBase):
     """test that infinite recursion due to incorrect backref assignments
@@ -1320,7 +1298,7 @@ class PendingBackrefTest(fixtures.ORMTest):
 
         p = Post("post 5")
 
-        # setting blog doesn't call 'posts' callable, calls with no fetch
+        # setting blog doesnt call 'posts' callable, calls with no fetch
         p.blog = b
         eq_(
             lazy_posts.mock_calls, [
@@ -1341,7 +1319,7 @@ class PendingBackrefTest(fixtures.ORMTest):
             ]
         )
 
-    def test_lazy_history(self):
+    def test_lazy_history_collection(self):
         Post, Blog, lazy_posts = self._fixture()
 
         p1, p2, p3 = Post("post 1"), Post("post 2"), Post("post 3")
@@ -1503,7 +1481,7 @@ class HistoryTest(fixtures.TestBase):
         class Foo(fixtures.BasicEntity):
             pass
         class Bar(fixtures.BasicEntity):
-            def __nonzero__(self):
+            def __bool__(self):
                 assert False
 
         instrumentation.register_class(Foo)
@@ -1513,6 +1491,12 @@ class HistoryTest(fixtures.TestBase):
         return Foo, Bar
 
     def _someattr_history(self, f, **kw):
+        passive = kw.pop('passive', None)
+        if passive is True:
+            kw['passive'] = attributes.PASSIVE_NO_INITIALIZE
+        elif passive is False:
+            kw['passive'] = attributes.PASSIVE_OFF
+
         return attributes.get_state_history(
                     attributes.instance_state(f),
                     'someattr', **kw)
@@ -1687,19 +1671,19 @@ class HistoryTest(fixtures.TestBase):
         Foo = self._fixture(uselist=True, useobject=True,
                                 active_history=True)
         f = Foo()
-        eq_(self._someattr_history(f, passive=True), ((), (), ()))
+        eq_(self._someattr_history(f, passive=True), (None, None, None))
 
     def test_scalar_obj_never_set(self):
         Foo = self._fixture(uselist=False, useobject=True,
                                 active_history=True)
         f = Foo()
-        eq_(self._someattr_history(f, passive=True), ((), (), ()))
+        eq_(self._someattr_history(f, passive=True), (None, None, None))
 
     def test_scalar_never_set(self):
         Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=True)
         f = Foo()
-        eq_(self._someattr_history(f, passive=True), ((), (), ()))
+        eq_(self._someattr_history(f, passive=True), (None, None, None))
 
     def test_scalar_active_set(self):
         Foo = self._fixture(uselist=False, useobject=False,
@@ -1795,6 +1779,24 @@ class HistoryTest(fixtures.TestBase):
         eq_(self._someattr_history(f), (['two'], (), ()))
 
 
+    def test_scalar_passive_flag(self):
+        Foo = self._fixture(uselist=False, useobject=False,
+                                active_history=True)
+        f = Foo()
+        f.someattr = 'one'
+        eq_(self._someattr_history(f), (['one'], (), ()))
+
+        self._commit_someattr(f)
+
+        state = attributes.instance_state(f)
+        state._expire_attribute_pre_commit(state.dict, 'someattr')
+
+        def scalar_loader(state, toload):
+            state.dict['someattr'] = 'one'
+        state.manager.deferred_scalar_loader = scalar_loader
+
+        eq_(self._someattr_history(f), ((), ['one'], ()))
+
 
     def test_scalar_inplace_mutation_set(self):
         Foo = self._fixture(uselist=False, useobject=False,
@@ -1850,7 +1852,6 @@ class HistoryTest(fixtures.TestBase):
         f.someattr = ['a']
         eq_(self._someattr_history(f), ([['a']], (), ()))
 
-
     def test_scalar_inplace_mutation_replace_self_flag_modified_set(self):
         Foo = self._fixture(uselist=False, useobject=False,
                                 active_history=False)
