@@ -2,7 +2,7 @@
 Primary key changing capabilities and passive/non-passive cascading updates.
 
 """
-from __future__ import with_statement
+
 from sqlalchemy.testing import eq_, ne_, \
                         assert_raises, assert_raises_message
 import sqlalchemy as sa
@@ -28,6 +28,7 @@ class NaturalPKTest(fixtures.MappedTest):
     # MySQL 5.5 on Windows crashes (the entire server, not the client)
     # if you screw around with ON UPDATE CASCADE type of stuff.
     __requires__ = 'skip_mysql_on_windows', 'on_update_or_deferrable_fks'
+    __backend__ = True
 
     @classmethod
     def define_tables(cls, metadata):
@@ -107,7 +108,7 @@ class NaturalPKTest(fixtures.MappedTest):
         users.update(values={User.username:'jack'}).execute(username='ed')
 
         # expire/refresh works off of primary key.  the PK is gone
-        # in this case so there's no way to look it up.  criterion-
+        # in this case so theres no way to look it up.  criterion-
         # based session invalidation could solve this [ticket:911]
         sess.expire(u1)
         assert_raises(sa.orm.exc.ObjectDeletedError, getattr, u1, 'username')
@@ -450,6 +451,7 @@ class NaturalPKTest(fixtures.MappedTest):
 
 class TransientExceptionTesst(_fixtures.FixtureTest):
     run_inserts = None
+    __backend__ = True
 
     def test_transient_exception(self):
         """An object that goes from a pk value to transient/pending
@@ -489,6 +491,7 @@ class ReversePKsTest(fixtures.MappedTest):
     """reverse the primary keys of two entities and ensure bookkeeping
     succeeds."""
 
+    __backend__ = True
 
     @classmethod
     def define_tables(cls, metadata):
@@ -496,7 +499,7 @@ class ReversePKsTest(fixtures.MappedTest):
             'user', metadata,
             Column('code', Integer, primary_key=True),
             Column('status', Integer, primary_key=True),
-            Column('username', Unicode(50), nullable=False),
+            Column('username', String(50), nullable=False),
             )
 
     @classmethod
@@ -516,11 +519,11 @@ class ReversePKsTest(fixtures.MappedTest):
 
         session = sa.orm.sessionmaker()()
 
-        a_published = User(1, PUBLISHED, u'a')
+        a_published = User(1, PUBLISHED, 'a')
         session.add(a_published)
         session.commit()
 
-        a_editable = User(1, EDITABLE, u'a')
+        a_editable = User(1, EDITABLE, 'a')
 
         session.add(a_editable)
         session.commit()
@@ -550,6 +553,7 @@ class SelfReferentialTest(fixtures.MappedTest):
     __unsupported_on__ = ('mssql', 'mysql')
 
     __requires__ = 'on_update_or_deferrable_fks',
+    __backend__ = True
 
     @classmethod
     def define_tables(cls, metadata):
@@ -654,6 +658,7 @@ class SelfReferentialTest(fixtures.MappedTest):
 
 class NonPKCascadeTest(fixtures.MappedTest):
     __requires__ = 'skip_mysql_on_windows', 'on_update_or_deferrable_fks'
+    __backend__ = True
 
     @classmethod
     def define_tables(cls, metadata):
@@ -759,6 +764,7 @@ class NonPKCascadeTest(fixtures.MappedTest):
 class CascadeToFKPKTest(fixtures.MappedTest, testing.AssertsCompiledSQL):
     """A primary key mutation cascades onto a foreign key that is itself a
     primary key."""
+    __backend__ = True
 
     @classmethod
     def define_tables(cls, metadata):
@@ -1012,6 +1018,7 @@ class JoinedInheritanceTest(fixtures.MappedTest):
     __unsupported_on__ = ('mssql',)
 
     __requires__ = 'skip_mysql_on_windows',
+    __backend__ = True
 
     @classmethod
     def define_tables(cls, metadata):
