@@ -633,7 +633,6 @@ class EagerTest(_fixtures.FixtureTest, testing.AssertsCompiledSQL):
             ], q.all())
         self.assert_sql_count(testing.db, go, 6)
 
-    @testing.fails_on('maxdb', 'FIXME: unknown')
     def test_limit(self):
         """Limit operations combined with lazy-load relationships."""
 
@@ -707,7 +706,6 @@ class EagerTest(_fixtures.FixtureTest, testing.AssertsCompiledSQL):
             eq_([User(id=7, address=Address(id=1))], l)
         self.assert_sql_count(testing.db, go, 2)
 
-    @testing.fails_on('maxdb', 'FIXME: unknown')
     def test_many_to_one(self):
         users, Address, addresses, User = (self.tables.users,
                                 self.classes.Address,
@@ -978,8 +976,7 @@ class OrderBySecondaryTest(fixtures.MappedTest):
         self.assert_sql_count(testing.db, go, 2)
 
 
-from .inheritance._poly_fixtures import _Polymorphic, Person, Engineer, \
-                Paperwork, Machine, MachineType, Company
+from .inheritance._poly_fixtures import _Polymorphic, Person, Engineer, Paperwork
 
 class BaseRelationFromJoinedSubclassTest(_Polymorphic):
     @classmethod
@@ -1135,118 +1132,7 @@ class BaseRelationFromJoinedSubclassTest(_Polymorphic):
             )
         )
 
-class SubRelationFromJoinedSubclassMultiLevelTest(_Polymorphic):
-    @classmethod
-    def define_tables(cls, metadata):
-        Table('companies', metadata,
-            Column('company_id', Integer,
-                primary_key=True,
-                test_needs_autoincrement=True),
-            Column('name', String(50)))
 
-        Table('people', metadata,
-            Column('person_id', Integer,
-                primary_key=True,
-                test_needs_autoincrement=True),
-            Column('company_id', Integer, ForeignKey('companies.company_id')),
-            Column('name', String(50)),
-            Column('type', String(30)))
-
-        Table('engineers', metadata,
-            Column('engineer_id', Integer, ForeignKey('people.person_id'),
-                            primary_key=True),
-            Column('primary_language', String(50)))
-
-        Table('machines', metadata,
-            Column('machine_id',
-                 Integer, primary_key=True,
-                 test_needs_autoincrement=True),
-            Column('name', String(50)),
-            Column('engineer_id', Integer, ForeignKey('engineers.engineer_id')),
-            Column('machine_type_id', Integer,
-                            ForeignKey('machine_type.machine_type_id')))
-
-        Table('machine_type', metadata,
-            Column('machine_type_id',
-                    Integer, primary_key=True,
-                        test_needs_autoincrement=True),
-            Column('name', String(50)))
-
-    @classmethod
-    def setup_mappers(cls):
-        companies = cls.tables.companies
-        people = cls.tables.people
-        engineers = cls.tables.engineers
-        machines = cls.tables.machines
-        machine_type = cls.tables.machine_type
-
-        mapper(Company, companies, properties={
-                'employees': relationship(Person, order_by=people.c.person_id)
-                })
-        mapper(Person, people,
-            polymorphic_on=people.c.type,
-            polymorphic_identity='person',
-            with_polymorphic='*')
-
-        mapper(Engineer, engineers,
-            inherits=Person,
-            polymorphic_identity='engineer', properties={
-                'machines': relationship(Machine,
-                                        order_by=machines.c.machine_id)
-            })
-
-        mapper(Machine, machines, properties={
-                'type': relationship(MachineType)
-                })
-        mapper(MachineType, machine_type)
-
-
-    @classmethod
-    def insert_data(cls):
-        c1 = cls._fixture()
-        sess = create_session()
-        sess.add(c1)
-        sess.flush()
-
-    @classmethod
-    def _fixture(cls):
-        mt1 = MachineType(name='mt1')
-        mt2 = MachineType(name='mt2')
-        return Company(
-                employees=[
-                    Engineer(
-                        name='e1',
-                        machines=[
-                            Machine(name='m1', type=mt1),
-                            Machine(name='m2', type=mt2)
-                        ]
-                    ),
-                    Engineer(
-                        name='e2',
-                        machines=[
-                            Machine(name='m3', type=mt1),
-                            Machine(name='m4', type=mt1)
-                        ]
-                    )
-                ])
-
-    def test_chained_subq_subclass(self):
-        s = Session()
-        q = s.query(Company).options(
-                        subqueryload(Company.employees.of_type(Engineer)),
-                        subqueryload(Company.employees.of_type(Engineer),
-                                                            Engineer.machines),
-                        subqueryload(Company.employees.of_type(Engineer),
-                                                            Engineer.machines,
-                                                            Machine.type)
-                    )
-
-        def go():
-            eq_(
-                q.all(),
-                [self._fixture()]
-            )
-        self.assert_sql_count(testing.db, go, 4)
 
 
 class SelfReferentialTest(fixtures.MappedTest):
@@ -1257,7 +1143,6 @@ class SelfReferentialTest(fixtures.MappedTest):
             Column('parent_id', Integer, ForeignKey('nodes.id')),
             Column('data', String(30)))
 
-    @testing.fails_on('maxdb', 'FIXME: unknown')
     def test_basic(self):
         nodes = self.tables.nodes
 
@@ -1422,7 +1307,6 @@ class SelfReferentialTest(fixtures.MappedTest):
             ]), d)
         self.assert_sql_count(testing.db, go, 3)
 
-    @testing.fails_on('maxdb', 'FIXME: unknown')
     def test_no_depth(self):
         """no join depth is set, so no eager loading occurs."""
 
@@ -1646,28 +1530,27 @@ class CyclicalInheritingEagerTestTwo(fixtures.DeclarativeMappedTest,
         q = ctx.attributes[('subquery',
                         (inspect(Director), inspect(Director).attrs.movies))]
         self.assert_compile(q,
-            "SELECT anon_1.movie_id AS anon_1_movie_id, "
-            "anon_1.persistent_id AS anon_1_persistent_id, "
-            "anon_1.movie_director_id AS anon_1_movie_director_id, "
-            "anon_1.movie_title AS anon_1_movie_title, "
-            "anon_2.director_id AS anon_2_director_id FROM "
-            "(SELECT director.id AS director_id FROM persistent JOIN director "
-            "ON persistent.id = director.id) AS anon_2 "
-            "JOIN (SELECT persistent.id AS persistent_id, movie.id AS movie_id, "
+            "SELECT movie.id AS movie_id, persistent.id AS persistent_id, "
             "movie.director_id AS movie_director_id, "
-            "movie.title AS movie_title FROM persistent JOIN movie "
-            "ON persistent.id = movie.id) AS anon_1 "
-            "ON anon_2.director_id = anon_1.movie_director_id "
-            "ORDER BY anon_2.director_id")
+            "movie.title AS movie_title, "
+            "anon_1.director_id AS anon_1_director_id "
+            "FROM (SELECT director.id AS director_id "
+                "FROM persistent JOIN director "
+                "ON persistent.id = director.id) AS anon_1 "
+            "JOIN (persistent JOIN movie ON persistent.id = movie.id) "
+            "ON anon_1.director_id = movie.director_id "
+            "ORDER BY anon_1.director_id",
+            dialect="default"
+        )
 
     def test_integrate(self):
         Director = self.classes.Director
         Movie = self.classes.Movie
 
         session = Session(testing.db)
-        rscott = Director(name=u"Ridley Scott")
-        alien = Movie(title=u"Alien")
-        brunner = Movie(title=u"Blade Runner")
+        rscott = Director(name="Ridley Scott")
+        alien = Movie(title="Alien")
+        brunner = Movie(title="Blade Runner")
         rscott.movies.append(brunner)
         rscott.movies.append(alien)
         session.add_all([rscott, alien, brunner])
@@ -1763,8 +1646,8 @@ class SubqueryloadDistinctTest(fixtures.DeclarativeMappedTest,
         q = (
             s.query(Movie)
             .options(
-                subqueryload(Movie.director),
-                subqueryload(Movie.director, Director.photos)
+                subqueryload(Movie.director)
+                .subqueryload(Director.photos)
             )
         )
         ctx = q._compile_context()
@@ -1817,16 +1700,17 @@ class SubqueryloadDistinctTest(fixtures.DeclarativeMappedTest,
         result = s.execute(q3)
         rows = result.fetchall()
         if expect_distinct:
-            eq_(set(tuple(r) for r in rows), set([
-                (1, u'/1.jpg', 1, 1),
-                (2, u'/2.jpg', 1, 1),
+            eq_(set(tuple(t) for t in rows), set([
+                (1, '/1.jpg', 1, 1),
+                (2, '/2.jpg', 1, 1),
             ]))
         else:
-            eq_(set(tuple(r) for r in rows), set([
-                (1, u'/1.jpg', 1, 1),
-                (2, u'/2.jpg', 1, 1),
-                (1, u'/1.jpg', 1, 1),
-                (2, u'/2.jpg', 1, 1),
+            # oracle might not order the way we expect here
+            eq_(set(tuple(t) for t in rows), set([
+                (1, '/1.jpg', 1, 1),
+                (2, '/2.jpg', 1, 1),
+                (1, '/1.jpg', 1, 1),
+                (2, '/2.jpg', 1, 1),
             ]))
 
 
@@ -1844,16 +1728,13 @@ class SubqueryloadDistinctTest(fixtures.DeclarativeMappedTest,
         Movie = self.classes.Movie
         Credit = self.classes.Credit
 
-        Credit.movie.property.distinct_target_key = False
-        Movie.director.property.distinct_target_key = False
-
         s = create_session()
 
         q = (
             s.query(Credit)
             .options(
-                subqueryload(Credit.movie),
-                subqueryload(Credit.movie, Movie.director)
+                subqueryload(Credit.movie)
+                .subqueryload(Movie.director)
             )
         )
 
@@ -1867,14 +1748,11 @@ class SubqueryloadDistinctTest(fixtures.DeclarativeMappedTest,
             ('subquery', (inspect(Movie), Movie.director.property))
         ]
 
-        # three rows due to dupe at Credit.movie level
-        # as well as Movie.director level
         result = s.execute(q3)
         eq_(
             result.fetchall(),
             [
                 (1, 'Woody Allen', 1), (1, 'Woody Allen', 1),
-                    (1, 'Woody Allen', 1)
             ]
         )
 
