@@ -3,6 +3,7 @@ from sqlalchemy import testing
 from sqlalchemy.testing import engines
 from sqlalchemy.testing.schema import Table, Column
 from test.orm import _fixtures
+from sqlalchemy import exc
 from sqlalchemy.testing import fixtures
 from sqlalchemy import Integer, String, ForeignKey, func
 from sqlalchemy.orm import mapper, relationship, backref, \
@@ -26,7 +27,7 @@ class AssertsUOW(object):
     def _assert_uow_size(self, session, expected ):
         uow = self._get_test_uow(session)
         postsort_actions = uow._generate_actions()
-        print postsort_actions
+        print(postsort_actions)
         eq_(len(postsort_actions), expected, postsort_actions)
 
 class UOWTest(_fixtures.FixtureTest,
@@ -125,12 +126,12 @@ class RudimentaryFlushTest(UOWTest):
                 CompiledSQL(
                     "UPDATE addresses SET user_id=:user_id WHERE "
                     "addresses.id = :addresses_id",
-                    lambda ctx: [{u'addresses_id': a1.id, 'user_id': None}]
+                    lambda ctx: [{'addresses_id': a1.id, 'user_id': None}]
                 ),
                 CompiledSQL(
                     "UPDATE addresses SET user_id=:user_id WHERE "
                     "addresses.id = :addresses_id",
-                    lambda ctx: [{u'addresses_id': a2.id, 'user_id': None}]
+                    lambda ctx: [{'addresses_id': a2.id, 'user_id': None}]
                 ),
                 CompiledSQL(
                     "DELETE FROM users WHERE users.id = :id",
@@ -235,12 +236,12 @@ class RudimentaryFlushTest(UOWTest):
                 CompiledSQL(
                     "UPDATE addresses SET user_id=:user_id WHERE "
                     "addresses.id = :addresses_id",
-                    lambda ctx: [{u'addresses_id': a1.id, 'user_id': None}]
+                    lambda ctx: [{'addresses_id': a1.id, 'user_id': None}]
                 ),
                 CompiledSQL(
                     "UPDATE addresses SET user_id=:user_id WHERE "
                     "addresses.id = :addresses_id",
-                    lambda ctx: [{u'addresses_id': a2.id, 'user_id': None}]
+                    lambda ctx: [{'addresses_id': a2.id, 'user_id': None}]
                 ),
                 CompiledSQL(
                     "DELETE FROM users WHERE users.id = :id",
@@ -1149,7 +1150,7 @@ class SingleCycleM2MTest(fixtures.MappedTest,
                     "nodes, node_to_nodes WHERE :param_1 = "
                     "node_to_nodes.right_node_id AND nodes.id = "
                     "node_to_nodes.left_node_id" ,
-                    lambda ctx:{u'param_1': n1.id},
+                    lambda ctx:{'param_1': n1.id},
                 ),
                 CompiledSQL(
                     "DELETE FROM node_to_nodes WHERE "
@@ -1284,7 +1285,7 @@ class BasicStaleChecksTest(fixtures.MappedTest):
             Column('data', Integer)
         )
 
-    def _fixture(self):
+    def _fixture(self, confirm_deleted_rows=True):
         parent, child = self.tables.parent, self.tables.child
 
         class Parent(fixtures.BasicEntity):
@@ -1295,8 +1296,8 @@ class BasicStaleChecksTest(fixtures.MappedTest):
         mapper(Parent, parent, properties={
             'child':relationship(Child, uselist=False,
                                     cascade="all, delete-orphan",
-                                    backref="parent")
-        })
+                                    backref="parent"),
+        }, confirm_deleted_rows=confirm_deleted_rows)
         mapper(Child, child)
         return Parent, Child
 
@@ -1317,7 +1318,8 @@ class BasicStaleChecksTest(fixtures.MappedTest):
             sess.flush
         )
 
-    def test_delete_multi_missing(self):
+    @testing.requires.sane_multi_rowcount
+    def test_delete_multi_missing_warning(self):
         Parent, Child = self._fixture()
         sess = Session()
         p1 = Parent(id=1, data=2, child=None)
@@ -1329,16 +1331,26 @@ class BasicStaleChecksTest(fixtures.MappedTest):
         sess.delete(p1)
         sess.delete(p2)
 
+        assert_raises_message(
+            exc.SAWarning,
+            "DELETE statement on table 'parent' expected to "
+                "delete 2 row\(s\); 0 were matched.",
+            sess.flush
+        )
+
+    def test_delete_multi_missing_allow(self):
+        Parent, Child = self._fixture(confirm_deleted_rows=False)
+        sess = Session()
+        p1 = Parent(id=1, data=2, child=None)
+        p2 = Parent(id=2, data=3, child=None)
+        sess.add_all([p1, p2])
         sess.flush()
 
-        # see issue #2403 - we *cannot* use rowcount here, as
-        # self-referential DELETE CASCADE could have deleted rows
-        #assert_raises_message(
-        #    orm_exc.StaleDataError,
-        #    "DELETE statement on table 'parent' expected to "
-        #        "delete 2 row\(s\); 0 were matched.",
-        #    sess.flush
-        #)
+        sess.execute(self.tables.parent.delete())
+        sess.delete(p1)
+        sess.delete(p2)
+
+        sess.flush()
 
 
 class BatchInsertsTest(fixtures.MappedTest, testing.AssertsExecutionResults):
