@@ -11,31 +11,14 @@ from sqlalchemy.orm import mapper, relationship, Session, \
 from sqlalchemy.testing import eq_, ne_, assert_raises, assert_raises_message
 from sqlalchemy.testing import fixtures
 from test.orm import _fixtures
-from sqlalchemy.testing import fixtures
-
-
-_uuids = [
-    '1fc614acbb904742a2990f86af6ded95',
-    '23e253786f4d491b9f9d6189dc33de9b',
-    'fc44910db37e43fd93e9ec8165b885cf',
-    '0187a1832b4249e6b48911821d86de58',
-    '778af6ea2fb74a009d8d2f5abe5dc29a',
-    '51a6ce031aff47e4b5f2895c4161f120',
-    '7434097cd319401fb9f15fa443ccbbbb',
-    '9bc548a8128e4a85ac18060bc3f4b7d3',
-    '59548715e3c440b7bcb96417d06f7930',
-    'd7647c7004734de196885ca2bd73adf8',
-    '70cef121d3ff48d39906b6d1ac77f41a',
-    'ee37a8a6430c466aa322b8a215a0dd70',
-    '782a5f04b4364a53a6fce762f48921c1',
-    'bef510f2420f4476a7629013ead237f5',
-    ]
+from sqlalchemy.testing.assertsql import AllOf, CompiledSQL
+import uuid
 
 def make_uuid():
-    """generate uuids even on Python 2.4 which has no 'uuid'"""
-    return _uuids.pop(0)
+    return uuid.uuid4().hex
 
 class VersioningTest(fixtures.MappedTest):
+    __backend__ = True
 
     @classmethod
     def define_tables(cls, metadata):
@@ -107,7 +90,7 @@ class VersioningTest(fixtures.MappedTest):
         else:
             s1.commit()
 
-        # new in 0.5 !  don't need to close the session
+        # new in 0.5 !  dont need to close the session
         f1 = s1.query(Foo).get(f1.id)
         f2 = s1.query(Foo).get(f2.id)
 
@@ -326,6 +309,7 @@ class VersioningTest(fixtures.MappedTest):
         )
 
 class ColumnTypeTest(fixtures.MappedTest):
+    __backend__ = True
 
     @classmethod
     def define_tables(cls, metadata):
@@ -366,6 +350,7 @@ class ColumnTypeTest(fixtures.MappedTest):
         s1.commit()
 
 class RowSwitchTest(fixtures.MappedTest):
+    __backend__ = True
     @classmethod
     def define_tables(cls, metadata):
         Table('p', metadata,
@@ -433,6 +418,7 @@ class RowSwitchTest(fixtures.MappedTest):
         session.commit()
 
 class AlternateGeneratorTest(fixtures.MappedTest):
+    __backend__ = True
     @classmethod
     def define_tables(cls, metadata):
         Table('p', metadata,
@@ -461,12 +447,12 @@ class AlternateGeneratorTest(fixtures.MappedTest):
                                 cls.classes.P)
 
         mapper(P, p, version_id_col=p.c.version_id,
-            version_id_generator=lambda x:make_uuid(),
+            version_id_generator=lambda x: make_uuid(),
             properties={
-            'c':relationship(C, uselist=False, cascade='all, delete-orphan')
+            'c': relationship(C, uselist=False, cascade='all, delete-orphan')
         })
         mapper(C, c, version_id_col=c.c.version_id,
-                    version_id_generator=lambda x:make_uuid(),
+                    version_id_generator=lambda x: make_uuid(),
         )
 
     @testing.emits_warning_on('+zxjdbc', r'.*does not support updated rowcount')
@@ -544,6 +530,7 @@ class InheritanceTwoVersionIdsTest(fixtures.MappedTest):
     versioning column.
 
     """
+    __backend__ = True
     @classmethod
     def define_tables(cls, metadata):
         Table('base', metadata,
@@ -643,3 +630,278 @@ class InheritanceTwoVersionIdsTest(fixtures.MappedTest):
             mapper,
             Sub, sub, inherits=Base,
                 version_id_col=sub.c.version_id)
+
+
+class ServerVersioningTest(fixtures.MappedTest):
+    run_define_tables = 'each'
+    __backend__ = True
+
+    @classmethod
+    def define_tables(cls, metadata):
+        from sqlalchemy.sql import ColumnElement
+        from sqlalchemy.ext.compiler import compiles
+        import itertools
+
+        counter = itertools.count(1)
+
+        class IncDefault(ColumnElement):
+            pass
+
+        @compiles(IncDefault)
+        def compile(element, compiler, **kw):
+            # cache the counter value on the statement
+            # itself so the assertsql system gets the same
+            # value when it compiles the statement a second time
+            stmt = compiler.statement
+            if hasattr(stmt, "_counter"):
+                return stmt._counter
+            else:
+                stmt._counter = str(next(counter))
+                return stmt._counter
+
+        Table('version_table', metadata,
+              Column('id', Integer, primary_key=True,
+                     test_needs_autoincrement=True),
+              Column('version_id', Integer, nullable=False,
+                            default=IncDefault(), onupdate=IncDefault()),
+              Column('value', String(40), nullable=False))
+
+    @classmethod
+    def setup_classes(cls):
+        class Foo(cls.Basic):
+            pass
+        class Bar(cls.Basic):
+            pass
+
+    def _fixture(self, expire_on_commit=True):
+        Foo, version_table = self.classes.Foo, self.tables.version_table
+
+        mapper(Foo, version_table,
+                version_id_col=version_table.c.version_id,
+                version_id_generator=False,
+            )
+
+        s1 = Session(expire_on_commit=expire_on_commit)
+        return s1
+
+    def test_insert_col(self):
+        sess = self._fixture()
+
+        f1 = self.classes.Foo(value='f1')
+        sess.add(f1)
+
+        statements = [
+                # note that the assertsql tests the rule against
+                # "default" - on a "returning" backend, the statement
+                # includes "RETURNING"
+                CompiledSQL(
+                    "INSERT INTO version_table (version_id, value) "
+                    "VALUES (1, :value)",
+                    lambda ctx: [{'value': 'f1'}]
+                )
+        ]
+        if not testing.db.dialect.implicit_returning:
+            # DBs without implicit returning, we must immediately
+            # SELECT for the new version id
+            statements.append(
+                CompiledSQL(
+                    "SELECT version_table.version_id AS version_table_version_id "
+                    "FROM version_table WHERE version_table.id = :param_1",
+                    lambda ctx: [{"param_1": 1}]
+                )
+            )
+        self.assert_sql_execution(testing.db, sess.flush, *statements)
+
+    def test_update_col(self):
+        sess = self._fixture()
+
+        f1 = self.classes.Foo(value='f1')
+        sess.add(f1)
+        sess.flush()
+
+        f1.value = 'f2'
+
+        statements = [
+                # note that the assertsql tests the rule against
+                # "default" - on a "returning" backend, the statement
+                # includes "RETURNING"
+                CompiledSQL(
+                    "UPDATE version_table SET version_id=2, value=:value "
+                    "WHERE version_table.id = :version_table_id AND "
+                    "version_table.version_id = :version_table_version_id",
+                    lambda ctx: [{"version_table_id": 1,
+                            "version_table_version_id": 1, "value": "f2"}]
+                )
+        ]
+        if not testing.db.dialect.implicit_returning:
+            # DBs without implicit returning, we must immediately
+            # SELECT for the new version id
+            statements.append(
+                CompiledSQL(
+                    "SELECT version_table.version_id AS version_table_version_id "
+                    "FROM version_table WHERE version_table.id = :param_1",
+                    lambda ctx: [{"param_1": 1}]
+                )
+            )
+        self.assert_sql_execution(testing.db, sess.flush, *statements)
+
+
+    def test_delete_col(self):
+        sess = self._fixture()
+
+        f1 = self.classes.Foo(value='f1')
+        sess.add(f1)
+        sess.flush()
+
+        sess.delete(f1)
+
+        statements = [
+                # note that the assertsql tests the rule against
+                # "default" - on a "returning" backend, the statement
+                # includes "RETURNING"
+                CompiledSQL(
+                    "DELETE FROM version_table "
+                    "WHERE version_table.id = :id AND "
+                    "version_table.version_id = :version_id",
+                    lambda ctx: [{"id": 1, "version_id": 1}]
+                )
+        ]
+        self.assert_sql_execution(testing.db, sess.flush, *statements)
+
+    def test_concurrent_mod_err_expire_on_commit(self):
+        sess = self._fixture()
+
+        f1 = self.classes.Foo(value='f1')
+        sess.add(f1)
+        sess.commit()
+
+        f1.value
+
+        s2 = Session()
+        f2 = s2.query(self.classes.Foo).first()
+        f2.value = 'f2'
+        s2.commit()
+
+        f1.value = 'f3'
+
+        assert_raises_message(
+            orm.exc.StaleDataError,
+            r"UPDATE statement on table 'version_table' expected to "
+            r"update 1 row\(s\); 0 were matched.",
+            sess.commit
+        )
+
+    def test_concurrent_mod_err_noexpire_on_commit(self):
+        sess = self._fixture(expire_on_commit=False)
+
+        f1 = self.classes.Foo(value='f1')
+        sess.add(f1)
+        sess.commit()
+
+        # here, we're not expired overall, so no load occurs and we
+        # stay without a version id, unless we've emitted
+        # a SELECT for it within the flush.
+        f1.value
+
+        s2 = Session(expire_on_commit=False)
+        f2 = s2.query(self.classes.Foo).first()
+        f2.value = 'f2'
+        s2.commit()
+
+        f1.value = 'f3'
+
+        assert_raises_message(
+            orm.exc.StaleDataError,
+            r"UPDATE statement on table 'version_table' expected to "
+            r"update 1 row\(s\); 0 were matched.",
+            sess.commit
+        )
+
+class ManualVersionTest(fixtures.MappedTest):
+    run_define_tables = 'each'
+    __backend__ = True
+
+    @classmethod
+    def define_tables(cls, metadata):
+        Table("a", metadata,
+                Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
+                Column('data', String(30)),
+                Column('vid', Integer)
+            )
+
+    @classmethod
+    def setup_classes(cls):
+        class A(cls.Basic):
+            pass
+
+
+    @classmethod
+    def setup_mappers(cls):
+        mapper(cls.classes.A, cls.tables.a,
+                        version_id_col=cls.tables.a.c.vid,
+                        version_id_generator=False)
+
+    def test_insert(self):
+        sess = Session()
+        a1 = self.classes.A()
+
+        a1.vid = 1
+        sess.add(a1)
+        sess.commit()
+
+        eq_(a1.vid, 1)
+
+    def test_update(self):
+        sess = Session()
+        a1 = self.classes.A()
+
+        a1.vid = 1
+        a1.data = 'd1'
+        sess.add(a1)
+        sess.commit()
+
+        a1.vid = 2
+        a1.data = 'd2'
+
+        sess.commit()
+
+        eq_(a1.vid, 2)
+
+    def test_update_concurrent_check(self):
+        sess = Session()
+        a1 = self.classes.A()
+
+        a1.vid = 1
+        a1.data = 'd1'
+        sess.add(a1)
+        sess.commit()
+
+        a1.vid = 2
+        sess.execute(self.tables.a.update().values(vid=3))
+        a1.data = 'd2'
+        assert_raises(
+            orm_exc.StaleDataError,
+            sess.commit
+        )
+
+    def test_update_version_conditional(self):
+        sess = Session()
+        a1 = self.classes.A()
+
+        a1.vid = 1
+        a1.data = 'd1'
+        sess.add(a1)
+        sess.commit()
+
+        # change the data and UPDATE without
+        # incrementing version id
+        a1.data = 'd2'
+        sess.commit()
+
+        eq_(a1.vid, 1)
+
+        a1.data = 'd3'
+        a1.vid = 2
+        sess.commit()
+
+        eq_(a1.vid, 2)
\ No newline at end of file
